#include "IVRStep.h"

IVRStep::IVRStep(std::string prompt, std::string pattern, std::string keep, IVRAction action)
{
    this->mPrompt = prompt;
    this->mPattern = pattern;
    this->mKeep = keep;
    this->mAction = action;
}

IVRStep::IVRStep()
{
    this->mPrompt = "";
    this->mPattern = "";
    this->mKeep = "";
    this->mAction = None;
}


IVRStep::~IVRStep(){
    for (std::list<IVRStep*>::iterator it=this->mSteps.begin();it!=this->mSteps.end();it++)
    {
        IVRStep* step = (*it);
        delete step;
    }

}

void IVRStep::setPrompt(std::string prompt)
{
    this->mPrompt = prompt;
}

void IVRStep::setPattern(std::string pattern)
{
    this->mPattern = pattern;
}

void IVRStep::setKeep(std::string keep)
{
    this->mKeep = keep;
}

void IVRStep::setAction(IVRAction action)
{
    this->mAction = action;
}

void IVRStep::addSubStep(IVRStep* step)
{
    this->mSteps.push_back(step);
}
std::string IVRStep::getPrompt()
{
    return this->mPrompt;
}

IVRAction IVRStep::getAction()
{
    if (!mSteps.size()) return NoSubStep;
    return this->mAction;
}

std::string IVRStep::getKeepName()
{
    return this->mKeep;
}

bool IVRStep::matches(std::string str)
{

    // Match variable size pattern. The subject string must end by '#'
    if (this->mPattern[0]=='*' && str[str.size()-1]=='#') return true;

    // Match patterns of same size with matching digits or 'x' or 'X'
    if (str.size() == this->mPattern.size())
    {
        bool match = true;
        for (int i=0;i<mPattern.size();i++)
        {
            if (str[i]!=mPattern[i] && mPattern[i]!='x' && mPattern[i]!='X')
            {
                match = false;
            }
        }
        return match;
    }

    return false;
    
}

IVRStep* IVRStep::onDigits(std::string digits)
{
    for (std::list<IVRStep*>::iterator it=this->mSteps.begin();it!=this->mSteps.end();it++)
    {
        IVRStep* step = (*it);
        if (step->matches(digits)) return step;
    }

    return this;
}

std::string IVRStep::toString()
{
    return this->mPrompt;
}

std::list<IVRStep*> IVRStep::getSteps()
{
    return this->mSteps;
}
